var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as signalR from "@microsoft/signalr";
import { nameof } from "ts-simple-nameof";
import { ConnectionResult, AuthorizationInfo, ConnectionState } from "./IExalusConnectionService";
import { TypedEvent } from "../TypedEvent";
import { Api } from '../Api';
import { LoggerService } from "./Logging/LoggerService";
import { Event } from "../Event";
import { ControllerConfigurationService } from "./Controller/ControllerConfigurationService";
import { WebApiCacheService } from './WebApi/WebApiCacheService';
export class ExalusConnectionService {
    constructor() {
        this._defaultPacketsBrokerAddress = "br1.tr7.pl";
        this._serversBrokerAddress = "https://broker.tr7.pl";
        this._address = "br1.tr7.pl";
        this._isEstabilished = false;
        this._timeout = 5000;
        this._dataReceivedEvent = new TypedEvent();
        this._pongReceivedEvent = new Event();
        this._authorizationReceivedEvent = new TypedEvent();
        this._registrationReceivedEvent = new TypedEvent();
        this._connectionStateChangedEvent = new TypedEvent();
        this._errorOccuredEvent = new TypedEvent();
        this._log = Api.Get(LoggerService);
        this._controllerConfiguration = null;
        this._cache = null;
        this._serviceName = nameof(ExalusConnectionService);
    }
    GetServiceName() {
        return this._serviceName;
    }
    InitializeConnection() {
        this._log.Debug(ExalusConnectionService, "Initializing Exalus Cloud connection service");
        this._controllerConfiguration = Api.Get(ControllerConfigurationService);
        this._cache = Api.Get(WebApiCacheService);
        if (this._connection)
            throw new Error("Connection already initialized");
        if (!this._address)
            throw new Error("Domain is not set");
        if (!this._serialId)
            throw new Error("SerialId is not set");
        if (!this._PIN)
            throw new Error("PIN is not set");
        this._connection = new signalR.HubConnectionBuilder()
            .withUrl(`https://${this._address}/broker`)
            .build();
    }
    SetServersBrokerAddress(address) {
        this._serversBrokerAddress = address;
    }
    SetDefaultPacketsBrokerAddress(address) {
        this._defaultPacketsBrokerAddress = address;
        this._address = address;
    }
    GetServerAddressAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield fetch(`${this._serversBrokerAddress}/api/connections/broker/whichserver/${this._serialId}`);
                if (result.ok) {
                    let address = yield result.text();
                    this._log.Debug(ExalusConnectionService, `Got server address: ${address}`);
                    return address;
                }
                else {
                    this._log.Error(ExalusConnectionService, `Failed to get server address, status: ${result.status}`);
                    return null;
                }
            }
            catch (ex) {
                this._log.Error(ExalusConnectionService, ex);
                return null;
            }
        });
    }
    ConnectAsync(address) {
        return __awaiter(this, void 0, void 0, function* () {
            this._address = address;
            return yield this.connectAsync();
        });
    }
    AuthorizeAsync(authorizationInfo) {
        let timeoutTime = 2000;
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            let timeoutId = 0;
            let sub = (result) => {
                window.clearTimeout(timeoutId);
                this._authorizationReceivedEvent.Unsubscribe(sub);
                resolve(result);
            };
            let onTimeout = () => {
                window.clearTimeout(timeoutId);
                this._authorizationReceivedEvent.Unsubscribe(sub);
                resolve(false);
            };
            timeoutId = window.setTimeout(onTimeout, timeoutTime);
            this._authorizationReceivedEvent.Subscribe(sub);
            yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.send("AuthorizeTo", authorizationInfo.SerialNumber, authorizationInfo.PIN));
        }));
    }
    ConnectAndAuthorizeAsync(authorizationInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._isEstabilished)
                return Promise.resolve(ConnectionResult.Connected);
            this._isEstabilished = true;
            this._serialId = authorizationInfo.SerialNumber;
            this._PIN = authorizationInfo.PIN;
            var packetsBroker = yield this.GetServerAddressAsync();
            if (packetsBroker == null)
                return Promise.resolve(ConnectionResult.FailedToConnectToServer);
            this._address = packetsBroker;
            let conResult = yield this.connectAsync();
            if (conResult != ConnectionResult.Connected)
                return conResult;
            return (yield this.AuthorizeAsync(authorizationInfo))
                ? ConnectionResult.Connected : ConnectionResult.AuthorizationFailed;
        });
    }
    connectAsync() {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            this.InitializeConnection();
            yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.start());
            (_b = this._connection) === null || _b === void 0 ? void 0 : _b.on("Pong", () => {
                this._log.Debug(ExalusConnectionService, "Pong received <-");
                this._pongReceivedEvent.Invoke();
            });
            (_c = this._connection) === null || _c === void 0 ? void 0 : _c.on("Registration", (data) => {
                var _a;
                (_a = this._registrationReceivedEvent) === null || _a === void 0 ? void 0 : _a.Invoke(data);
            });
            (_d = this._connection) === null || _d === void 0 ? void 0 : _d.on("Authorization", (data) => {
                var _a;
                this._log.Debug(ExalusConnectionService, `Received authorization response: ${data}`);
                (_a = this._authorizationReceivedEvent) === null || _a === void 0 ? void 0 : _a.Invoke(data);
            });
            (_e = this._connection) === null || _e === void 0 ? void 0 : _e.on("SendError", (sender, data) => __awaiter(this, void 0, void 0, function* () {
                if (sender.startsWith("NotAuthorized:")) {
                    this._log.Warning(ExalusConnectionService, "Failed to authorize!");
                    yield this.AuthorizeAsync(new AuthorizationInfo(this._serialId, this._PIN));
                }
                else {
                    this._log.Error(ExalusConnectionService, `Server returned error: ${sender} ${data}`);
                    this._errorOccuredEvent.Invoke([sender, data]);
                }
            }));
            let dataReceivedEvent = this._dataReceivedEvent;
            (_f = this._connection) === null || _f === void 0 ? void 0 : _f.on("Data", (sender, data) => {
                const d = JSON.parse(data);
                this._log.Debug(ExalusConnectionService, `Received data:\n${JSON.stringify(d, null, 2)}`);
                dataReceivedEvent.Invoke(d);
            });
            const connectionEvent = this._connectionStateChangedEvent;
            switch ((_g = this._connection) === null || _g === void 0 ? void 0 : _g.state) {
                case signalR.HubConnectionState.Connected:
                    connectionEvent.Invoke(ConnectionState.Connected);
                    return ConnectionResult.Connected;
                case signalR.HubConnectionState.Connecting:
                    connectionEvent.Invoke(ConnectionState.Connecting);
                    return ConnectionResult.Connected;
                case signalR.HubConnectionState.Disconnected:
                    connectionEvent.Invoke(ConnectionState.Disconnected);
                    return ConnectionResult.FailedToConnect;
                case signalR.HubConnectionState.Disconnecting:
                    connectionEvent.Invoke(ConnectionState.Disconnecting);
                    return ConnectionResult.FailedToConnect;
                case signalR.HubConnectionState.Reconnecting:
                    connectionEvent.Invoke(ConnectionState.Reconnecting);
                    return ConnectionResult.FailedToConnect;
            }
            return ConnectionResult.FailedToConnect;
        });
    }
    DisconnectAsync() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            yield ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.stop());
        });
    }
    IsConnected() {
        var _a;
        return ((_a = this._connection) === null || _a === void 0 ? void 0 : _a.state) === signalR.HubConnectionState.Connected;
    }
    SendAndWaitForResponseAsync(dataFrame, timeout, useCache, logTransmission = true) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (useCache) {
                if (!(yield ((_a = this._controllerConfiguration) === null || _a === void 0 ? void 0 : _a.DidCofigurationChangeAsync()))) {
                    let res = (_b = this._cache) === null || _b === void 0 ? void 0 : _b.GetCache(dataFrame);
                    if (res !== null) {
                        return Promise.resolve(res);
                    }
                }
            }
            let timeoutId = 0;
            const time = new Date();
            const startTime = time.getDate();
            if (!this.IsConnected())
                throw new Error("Connection is not established");
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                let onReceivedFrame = (receivedFrame) => {
                    var _a;
                    if ((receivedFrame === null || receivedFrame === void 0 ? void 0 : receivedFrame.TransactionId) == dataFrame.TransactionId) {
                        window.clearTimeout(timeoutId);
                        this._dataReceivedEvent.Unsubscribe(onReceivedFrame);
                        const difference = startTime - (new Date()).getDate();
                        if (logTransmission)
                            this._log.Debug(ExalusConnectionService, `Received response for: ${receivedFrame === null || receivedFrame === void 0 ? void 0 : receivedFrame.Resource} ${receivedFrame === null || receivedFrame === void 0 ? void 0 : receivedFrame.Method} id: ${receivedFrame === null || receivedFrame === void 0 ? void 0 : receivedFrame.TransactionId} in ${difference}ms`);
                        if (useCache)
                            (_a = this._cache) === null || _a === void 0 ? void 0 : _a.Cache(receivedFrame);
                        resolve(receivedFrame);
                    }
                };
                this._dataReceivedEvent.Subscribe(onReceivedFrame);
                timeoutId = window.setTimeout(() => {
                    this._dataReceivedEvent.Unsubscribe(onReceivedFrame);
                    let errorMessage = `Response timeout, resource: ${dataFrame.Resource} method: ${dataFrame.Method} transaction id: ${dataFrame.TransactionId}`;
                    this._log.Error(ExalusConnectionService, errorMessage);
                    reject(new signalR.TimeoutError(errorMessage));
                }, timeout);
                if (!(yield this.SendAsync(dataFrame, logTransmission))) {
                    let errorMessage = `Failed to send request, resource: ${dataFrame.Resource} method: ${dataFrame.Method} transaction id: ${dataFrame.TransactionId}`;
                    reject(new Error(errorMessage));
                }
            }));
        });
    }
    SendAsync(dataFrame, logTransmission = false) {
        if (!this.IsConnected())
            throw new Error("Connection is not established");
        return new Promise((resolve) => {
            var _a;
            if (logTransmission)
                this._log.Debug(ExalusConnectionService, `Send DataFrame ${dataFrame.Resource} ${dataFrame.Method} transaction id: ${dataFrame.TransactionId} `);
            (_a = this._connection) === null || _a === void 0 ? void 0 : _a.invoke("SendTo", this._serialId, dataFrame).then(() => {
                resolve(true);
            }).catch(err => {
                this._log.Error(ExalusConnectionService, `${err}`);
                resolve(false);
            });
        });
    }
    OnDataReceivedEvent() {
        return this._dataReceivedEvent;
    }
    OnConnectionStateChangedEvent() {
        return this._connectionStateChangedEvent;
    }
    OnErrorOccuredEvent() {
        return this._errorOccuredEvent;
    }
}
//# sourceMappingURL=ExalusConnectionService.js.map